@@ -2,25 +2,25 @@ import sys
 from concurrent.futures import ThreadPoolExecutor
 from functools import wraps, partial
 from io import StringIO
+from typing import (
+    Any, Callable, Dict, List, Set, Tuple, Union, TypeVar, Sequence, NamedTuple, Iterable,
+    Container, Generic)
 
 import pytest
 
 from typeguard import (
-    typechecked, check_argument_types, qualified_name, TypeChecker, TypeWarning, function_name)
+    typechecked, check_argument_types, qualified_name, TypeChecker, TypeWarning, function_name,
+    check_type)
 
 try:
-    from backports.typing import (
-        Any, Callable, Dict, List, Set, Tuple, Union, TypeVar, Sequence, NamedTuple, Iterable,
-        Container, Type, Generic)
+    from typing import Type
 except ImportError:
-    from typing import (
-        Any, Callable, Dict, List, Set, Tuple, Union, TypeVar, Sequence, NamedTuple, Iterable,
-        Container, Generic)
+    Type = List  # don't worry, Type is not actually used if this happens!
 
-    try:
-        from typing import Type
-    except ImportError:
-        Type = List  # don't worry, Type is not actually used if this happens!
+try:
+    from typing import Collection
+except ImportError:
+    Collection = None
 
 
 class Parent:
@@ -45,6 +45,15 @@ def test_function_name():
     assert function_name(function_name) == 'typeguard.function_name'
 
 
+def test_check_type_no_memo():
+    check_type('foo', [1], List[int])
+
+
+def test_check_type_no_memo_fail():
+    pytest.raises(TypeError, check_type, 'foo', ['a'], List[int]).\
+        match('type of foo\[0\] must be int; got str instead')
+
+
 class TestCheckArgumentTypes:
     def test_any_type(self):
         def foo(a: Any):
@@ -119,6 +128,15 @@ class TestCheckArgumentTypes:
 
         foo(SomeClass)
 
+    def test_callable_plain(self):
+        def foo(a: Callable):
+            assert check_argument_types()
+
+        def callback(a):
+            pass
+
+        foo(callback)
+
     def test_callable_partial_class(self):
         """
         Test that passing a bound method as a callable does not count the "self" argument "a"gainst
@@ -324,6 +342,13 @@ class TestCheckArgumentTypes:
 
         foo(value)
 
+    def test_union_typing_type(self):
+        def foo(a: Union[str, Collection]):
+            assert check_argument_types()
+
+        with pytest.raises(TypeError):
+            foo(1)
+
     @pytest.mark.parametrize('value', [6.5, b'aa'])
     def test_union_fail(self, value):
         def foo(a: Union[str, int]):
@@ -346,6 +371,15 @@ class TestCheckArgumentTypes:
 
         foo(*values)
 
+    def test_typevar_constraints_fail_typing_type(self):
+        T = TypeVar('T', int, Collection)
+
+        def foo(a: T, b: T):
+            assert check_argument_types()
+
+        with pytest.raises(TypeError):
+            foo('aa', 'bb')
+
     def test_typevar_constraints_fail(self):
         T = TypeVar('T', int, str)
 
@@ -442,20 +476,21 @@ class TestCheckArgumentTypes:
         assert str(exc.value) == ('type of argument "a" must be test_typeguard.Child; '
                                   'got test_typeguard.Parent instead')
 
-    @pytest.mark.parametrize('values', [
-        ('x', 1.2, 'y'),
-        (2, 'x', None)
-    ], ids=['declared', 'default'])
-    def test_default_argument_type(self, values):
-        """
-        Test that the type of the default argument is also accepted even if it does not match the
-        declared type of the argument.
-
-        """
-        def foo(a: str=1, b: float='x', c: str=None):
+    def test_mismatching_default_type(self):
+        def foo(a: str = 1):
             assert check_argument_types()
 
-        foo(*values)
+        pytest.raises(TypeError, foo).match('type of argument "a" must be str; got int instead')
+
+    def test_implicit_default_none(self):
+        """
+        Test that if the default value is ``None``, a ``None`` argument can be passed.
+
+        """
+        def foo(a: str=None):
+            assert check_argument_types()
+
+        foo()
 
     def test_generator(self):
         """Test that argument type checking works in a generator function too."""
@@ -504,6 +539,30 @@ class TestCheckArgumentTypes:
 
         foo(FooGeneric[str]())
 
+    def test_newtype(self):
+        try:
+            from typing import NewType
+        except ImportError:
+            pytest.skip('Skipping newtype test since no NewType in current typing library')
+
+        myint = NewType("myint", int)
+
+        def foo(a: myint) -> int:
+            assert check_argument_types()
+            return 42
+
+        assert foo(1) == 42
+        exc = pytest.raises(TypeError, foo, "a")
+        assert str(exc.value) == 'type of argument "a" must be int; got str instead'
+
+    @pytest.mark.skipif(Collection is None, reason='typing.Collection is not available')
+    def test_collection(self):
+        def foo(a: Collection):
+            assert check_argument_types()
+
+        pytest.raises(TypeError, foo, True).match(
+            'type of argument "a" must be collections.abc.Collection; got bool instead')
+
 
 class TestTypeChecked:
     def test_typechecked(self):
