@@ -1,11 +1,13 @@
-import collections
+__all__ = ('typechecked', 'check_argument_types', 'check_type', 'TypeWarning', 'TypeChecker')
+
+import collections.abc
 import gc
 import inspect
 import sys
 import threading
 from collections import OrderedDict
 from functools import wraps, partial
-from inspect import Parameter, isclass
+from inspect import Parameter, isclass, isfunction
 from traceback import extract_stack, print_stack
 from types import CodeType, FunctionType  # noqa
 from typing import (Callable, Any, Union, Dict, List, TypeVar, Tuple, Set, Sequence,
@@ -18,16 +20,6 @@ try:
 except ImportError:
     Type = None
 
-try:
-    from inspect import unwrap
-except ImportError:
-    def unwrap(func):
-        while hasattr(func, '__wrapped__'):
-            func = func.__wrapped__
-
-        return func
-
-__all__ = ('typechecked', 'check_argument_types', 'TypeWarning', 'TypeChecker')
 _type_hints_map = WeakKeyDictionary()  # type: Dict[FunctionType, Dict[str, Any]]
 _functions_map = WeakValueDictionary()  # type: Dict[CodeType, FunctionType]
 
@@ -54,10 +46,11 @@ class _CallMemo:
             self.type_hints = _type_hints_map[func] = OrderedDict()
             for name, parameter in self.signature.parameters.items():
                 if name in hints:
-                    # If an argument has a default value, its type should be accepted as well
                     annotated_type = hints[name]
-                    if parameter.default is not Parameter.empty:
-                        annotated_type = Union[annotated_type, type(parameter.default)]
+
+                    # PEP 428 discourages it by MyPy does not complain
+                    if parameter.default is None:
+                        annotated_type = Optional[annotated_type]
 
                     if parameter.kind == Parameter.VAR_POSITIONAL:
                         self.type_hints[name] = Tuple[annotated_type, ...]
@@ -70,6 +63,11 @@ class _CallMemo:
                 self.type_hints['return'] = hints['return']
 
 
+def get_type_name(type_):
+    # typing.* types don't have a __name__ on Python 3.7+
+    return getattr(type_, '__name__', None) or type_._name
+
+
 def find_function(frame) -> Optional[Callable]:
     """
     Return a function object from the garbage collector that matches the frame's code object.
@@ -96,6 +94,8 @@ def find_function(frame) -> Optional[Callable]:
         # Cache the result for future lookups
         if func is not None:
             _functions_map[frame.f_code] = func
+        else:
+            raise LookupError('target function not found')
 
     return func
 
@@ -127,11 +127,11 @@ def function_name(func: FunctionType) -> str:
     return qualname if module == 'builtins' else '{}.{}'.format(module, qualname)
 
 
-def check_callable(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_callable(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     if not callable(value):
         raise TypeError('{} must be a callable'.format(argname))
 
-    if isinstance(expected_type.__args__, tuple):
+    if expected_type.__args__:
         try:
             signature = inspect.signature(value)
         except (TypeError, ValueError):
@@ -175,7 +175,7 @@ def check_callable(argname: str, value, expected_type, memo: _CallMemo) -> None:
                                                          num_mandatory_args))
 
 
-def check_dict(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_dict(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     if not isinstance(value, dict):
         raise TypeError('type of {} must be a dict; got {} instead'.
                         format(argname, qualified_name(value)))
@@ -186,7 +186,7 @@ def check_dict(argname: str, value, expected_type, memo: _CallMemo) -> None:
         check_type('{}[{!r}]'.format(argname, k), v, value_type, memo)
 
 
-def check_list(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_list(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     if not isinstance(value, list):
         raise TypeError('type of {} must be a list; got {} instead'.
                         format(argname, qualified_name(value)))
@@ -197,7 +197,7 @@ def check_list(argname: str, value, expected_type, memo: _CallMemo) -> None:
             check_type('{}[{}]'.format(argname, i), v, value_type, memo)
 
 
-def check_sequence(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_sequence(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     if not isinstance(value, collections.Sequence):
         raise TypeError('type of {} must be a sequence; got {} instead'.
                         format(argname, qualified_name(value)))
@@ -208,7 +208,7 @@ def check_sequence(argname: str, value, expected_type, memo: _CallMemo) -> None:
             check_type('{}[{}]'.format(argname, i), v, value_type, memo)
 
 
-def check_set(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_set(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     if not isinstance(value, collections.Set):
         raise TypeError('type of {} must be a set; got {} instead'.
                         format(argname, qualified_name(value)))
@@ -219,7 +219,7 @@ def check_set(argname: str, value, expected_type, memo: _CallMemo) -> None:
             check_type('elements of {}'.format(argname), v, value_type, memo)
 
 
-def check_tuple(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_tuple(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     # Specialized check for NamedTuples
     if hasattr(expected_type, '_field_types'):
         if not isinstance(value, expected_type):
@@ -259,7 +259,7 @@ def check_tuple(argname: str, value, expected_type, memo: _CallMemo) -> None:
             check_type('{}[{}]'.format(argname, i), element, element_type, memo)
 
 
-def check_union(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_union(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     if hasattr(expected_type, '__union_params__'):
         # Python 3.5
         union_params = expected_type.__union_params__
@@ -274,16 +274,20 @@ def check_union(argname: str, value, expected_type, memo: _CallMemo) -> None:
         except TypeError:
             pass
 
-    typelist = ', '.join(t.__name__ for t in union_params)
+    typelist = ', '.join(get_type_name(t) for t in union_params)
     raise TypeError('type of {} must be one of ({}); got {} instead'.
                     format(argname, typelist, qualified_name(value)))
 
 
-def check_class(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_class(argname: str, value, expected_type, memo: Optional[_CallMemo]) -> None:
     if not isclass(value):
         raise TypeError('type of {} must be a type; got {} instead'.format(
             argname, qualified_name(value)))
 
+    # Needed on Python 3.7+
+    if expected_type is Type:
+        return
+
     expected_class = expected_type.__args__[0] if expected_type.__args__ else None
     if expected_class:
         if isinstance(expected_class, TypeVar):
@@ -293,8 +297,11 @@ def check_class(argname: str, value, expected_type, memo: _CallMemo) -> None:
                 argname, qualified_name(expected_class), qualified_name(value)))
 
 
-def check_typevar(argname: str, value, typevar: TypeVar, memo: _CallMemo,
+def check_typevar(argname: str, value, typevar: TypeVar, memo: Optional[_CallMemo],
                   subclass_check: bool = False) -> None:
+    if memo is None:
+        raise TypeError('encountered a TypeVar but a call memo was not provided')
+
     bound_type = memo.typevars.get(typevar, typevar.__bound__)
     value_type = value if subclass_check else type(value)
     subject = argname if subclass_check else 'type of ' + argname
@@ -302,7 +309,7 @@ def check_typevar(argname: str, value, typevar: TypeVar, memo: _CallMemo,
         # The type variable hasn't been bound yet -- check that the given value matches the
         # constraints of the type variable, if any
         if typevar.__constraints__ and value_type not in typevar.__constraints__:
-            typelist = ', '.join(t.__name__ for t in typevar.__constraints__
+            typelist = ', '.join(get_type_name(t) for t in typevar.__constraints__
                                  if t is not object)
             raise TypeError('{} must be one of ({}); got {} instead'.
                             format(subject, typelist, qualified_name(value_type)))
@@ -338,10 +345,19 @@ def check_number(argname: str, value, expected_type):
 
 # Equality checks are applied to these
 origin_type_checkers = {
+    Callable: check_callable,
+    collections.abc.Callable: check_callable,
+    dict: check_dict,
     Dict: check_dict,
+    list: check_list,
     List: check_list,
     Sequence: check_sequence,
+    collections.abc.Sequence: check_sequence,
+    set: check_set,
     Set: check_set,
+    tuple: check_tuple,
+    Tuple: check_tuple,
+    type: check_class,
     Union: check_union
 }
 _subclass_check_unions = hasattr(Union, '__union_set_params__')
@@ -349,7 +365,7 @@ if Type is not None:
     origin_type_checkers[Type] = check_class
 
 
-def check_type(argname: str, value, expected_type, memo: _CallMemo) -> None:
+def check_type(argname: str, value, expected_type, memo: Optional[_CallMemo] = None) -> None:
     """
     Ensure that ``value`` matches ``expected_type``.
 
@@ -369,17 +385,18 @@ def check_type(argname: str, value, expected_type, memo: _CallMemo) -> None:
         # Only happens on < 3.6
         expected_type = type(None)
 
-    if isclass(expected_type):
-        origin_type = getattr(expected_type, '__origin__', None)
-        if origin_type is not None:
-            checker_func = origin_type_checkers.get(origin_type)
-            if checker_func:
-                checker_func(argname, value, expected_type, memo)
-                return
-
+    origin_type = getattr(expected_type, '__origin__', None)
+    if origin_type is not None:
+        checker_func = origin_type_checkers.get(origin_type)
+        if checker_func:
+            checker_func(argname, value, expected_type, memo)
+        else:
+            check_type(argname, value, origin_type, memo)
+    elif isclass(expected_type):
         if issubclass(expected_type, Tuple):
             check_tuple(argname, value, expected_type, memo)
         elif issubclass(expected_type, Callable) and hasattr(expected_type, '__args__'):
+            # Needed on Python 3.5.0 to 3.5.2
             check_callable(argname, value, expected_type, memo)
         elif issubclass(expected_type, (float, complex)):
             check_number(argname, value, expected_type)
@@ -397,40 +414,52 @@ def check_type(argname: str, value, expected_type, memo: _CallMemo) -> None:
     elif isinstance(expected_type, TypeVar):
         # Only happens on < 3.6
         check_typevar(argname, value, expected_type, memo)
-    elif getattr(expected_type, '__origin__', None) is Union:
-        # Only happens on 3.6+
-        check_union(argname, value, expected_type, memo)
+    elif (isfunction(expected_type) and
+            getattr(expected_type, "__module__", None) == "typing" and
+            getattr(expected_type, "__qualname__", None).startswith("NewType.") and
+            hasattr(expected_type, "__supertype__")):
+        # typing.NewType, should check against supertype (recursively)
+        return check_type(argname, value, expected_type.__supertype__, memo)
 
 
-def check_return_type(retval, memo: _CallMemo) -> bool:
+def check_return_type(retval, memo: Optional[_CallMemo]) -> bool:
     if 'return' in memo.type_hints:
-        check_type('the return value', retval, memo.type_hints['return'], memo)
+        try:
+            check_type('the return value', retval, memo.type_hints['return'], memo)
+        except TypeError as exc:  # suppress unnecessarily long tracebacks
+            raise TypeError(exc) from None
 
     return True
 
 
-def check_argument_types(memo: _CallMemo = None) -> bool:
+def check_argument_types(memo: Optional[_CallMemo] = None) -> bool:
     """
     Check that the argument values match the annotated types.
 
     Unless both ``args`` and ``kwargs`` are provided, the information will be retrieved from
     the previous stack frame (ie. from the function that called this).
 
-    :param func: the callable to check the arguments against
     :return: ``True``
     :raises TypeError: if there is an argument type mismatch
 
     """
     if memo is None:
         frame = inspect.currentframe().f_back
-        func = find_function(frame)
+        try:
+            func = find_function(frame)
+        except LookupError:
+            return True  # This can happen with the Pydev/PyCharm debugger extension installed
+
         memo = _CallMemo(func, frame)
 
     for argname, expected_type in memo.type_hints.items():
         if argname != 'return' and argname in memo.arguments:
             value = memo.arguments[argname]
-            description = 'argument "{}"'.format(argname, memo.func_name)
-            check_type(description, value, expected_type, memo)
+            description = 'argument "{}"'.format(argname)
+            try:
+                check_type(description, value, expected_type, memo)
+            except TypeError as exc:  # suppress unnecessarily long tracebacks
+                raise TypeError(exc) from None
 
     return True
 
@@ -483,7 +512,7 @@ class TypeWarning(UserWarning):
 
     __slots__ = ('func', 'event', 'message', 'frame')
 
-    def __init__(self, memo: _CallMemo, event: str, frame,
+    def __init__(self, memo: Optional[_CallMemo], event: str, frame,
                  exception: TypeError):  # pragma: no cover
         self.func = memo.func
         self.event = event
@@ -508,7 +537,7 @@ class TypeWarning(UserWarning):
 
     def print_stack(self, file: TextIO = None, limit: int = None) -> None:
         """
-        Print the traceback from the stack frame where the target function was ran.
+        Print the traceback from the stack frame where the target function was run.
 
         :param file: an open file to print to (prints to stdout if omitted)
         :param limit: the maximum number of stack frames to print
